@@ -13,8 +13,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/IRIX/osi_vfsops.c,v 1.14 2004/08/19 02:19:16 kolya Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -34,8 +32,6 @@ mutex_t afs_init_kern_lock;
 
 #define SYS_setgroups SGI_SETGROUPS
 
-int (*nfs_rfsdisptab_v2) () = NULL;
-
 int afs_fstype;
 lock_t afs_rxlock;
 
@@ -57,11 +53,7 @@ extern void afs_mpservice(void *);
 extern struct vfsops Afs_vfsops, *afs_vfsopsp;
 extern struct vnodeops Afs_vnodeops, *afs_vnodeopsp;
 extern void (*afsidestroyp) (struct inode *);
-extern void (*afsdptoipp) (struct efs_dinode *, struct inode *);
-extern void (*afsiptodpp) (struct inode *, struct efs_dinode *);
 extern void afsidestroy(struct inode *);
-extern void afsdptoip(struct efs_dinode *, struct inode *);
-extern void afsiptodp(struct inode *, struct efs_dinode *);
 extern int (*idbg_prafsnodep) (vnode_t *);
 extern int (*idbg_afsvfslistp) (void);
 extern int idbg_prafsnode(vnode_t *);
@@ -93,8 +85,6 @@ Afs_init(struct vfssw *vswp, int fstype)
     afs_vnodeopsp = &Afs_vnodeops;
     afs_vfsopsp = &Afs_vfsops;
     afsidestroyp = afsidestroy;
-    afsiptodpp = afsiptodp;
-    afsdptoipp = afsdptoip;
     idbg_prafsnodep = idbg_prafsnode;
     idbg_afsvfslistp = idbg_afsvfslist;
     NewSystemCall(AFS_SYSCALL, Afs_syscall, 6);
@@ -194,11 +184,6 @@ afs_mount(struct vfs *afsp, vnode_t * mvp, struct mounta *uap,
     afsp->vfs_fstype = afs_fstype;
     afsp->vfs_dev = 0xbabebabe;	/* XXX this should be unique */
 
-#ifndef	AFS_NONFSTRANS
-    if (nfs_rfsdisptab_v2)
-	afs_xlatorinit_v2(nfs_rfsdisptab_v2);
-    afs_xlatorinit_v3();
-#endif
     return 0;
 }
 
@@ -207,10 +192,9 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
      int flags;
      cred_t *cr;
 {
-    extern int afs_afs_cold_shutdown;
     struct vcache *tvc;
     vnode_t *vp, *rootvp = NULL;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int error, fv_slept;
     OSI_VFS_CONVERT(afsp);
@@ -225,11 +209,12 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
      * EBUSY if any still in use
      */
     ObtainWriteLock(&afs_xvcache, 172);
+ retry:
     for (tq = VLRU.prev; tq != &VLRU; tq = uq) {
 	tvc = QTOV(tq);
 	uq = QPrev(tq);
 	vp = (vnode_t *) tvc;
-	if (error = afs_FlushVCache(tvc, &fv_slept))
+	if (error = afs_FlushVCache(tvc, &fv_slept)) {
 	    if (vp->v_flag & VROOT) {
 		rootvp = vp;
 		continue;
@@ -237,6 +222,10 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
 		ReleaseWriteLock(&afs_xvcache);
 		return error;
 	    }
+	}
+	if (fv_slept) {
+	    goto retry;
+	}
     }
 
     /*
@@ -269,13 +258,13 @@ afs_root(OSI_VFS_ARG(afsp), avpp)
     OSI_VFS_DECL(afsp);
      struct vnode **avpp;
 {
-    register afs_int32 code = 0;
+    afs_int32 code = 0;
     struct vrequest treq;
-    register struct vcache *tvp = 0;
+    struct vcache *tvp = 0;
     OSI_VFS_CONVERT(afsp);
 
     AFS_STATCNT(afs_root);
-    if (afs_globalVp && (afs_globalVp->states & CStatd)) {
+    if (afs_globalVp && (afs_globalVp->f.states & CStatd)) {
 	tvp = afs_globalVp;
     } else {
 	if (afs_globalVp) {
@@ -376,7 +365,7 @@ afs_sync(OSI_VFS_DECL(afsp),
     struct vcache *tvc;
     struct vnode *vp;
     afs_uint32 lvcachegen;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int s;
     OSI_VFS_CONVERT(afsp);
@@ -460,7 +449,7 @@ afs_sync(OSI_VFS_DECL(afsp),
 
 	AFS_GUNLOCK();
 	if (flags & SYNC_CLOSE) {
-	    PFLUSHINVALVP(vp, (off_t) 0, (off_t) tvc->m.Length);
+	    PFLUSHINVALVP(vp, (off_t) 0, (off_t) tvc->f.m.Length);
 	}
 #ifdef AFS_SGI61_ENV
 	else if (flags & SYNC_PDFLUSH) {
@@ -473,18 +462,18 @@ afs_sync(OSI_VFS_DECL(afsp),
 
 	if ((flags & SYNC_DELWRI) && AFS_VN_DIRTY(vp)) {
 #ifdef AFS_SGI61_ENV
-	    PFLUSHVP(vp, (off_t) tvc->m.Length,
+	    PFLUSHVP(vp, (off_t) tvc->f.m.Length,
 		     (flags & SYNC_WAIT) ? 0 : B_ASYNC, error);
 #else /* AFS_SGI61_ENV */
 	    if (flags & SYNC_WAIT)
 		/* push all and wait */
-		PFLUSHVP(vp, (off_t) tvc->m.Length, (off_t) 0, error);
+		PFLUSHVP(vp, (off_t) tvc->f.m.Length, (off_t) 0, error);
 	    else if (flags & SYNC_BDFLUSH) {
 		/* push oldest */
 		error = pdflush(vp, B_ASYNC);
 	    } else {
 		/* push all but don't wait */
-		PFLUSHVP(vp, (off_t) tvc->m.Length, (off_t) B_ASYNC, error);
+		PFLUSHVP(vp, (off_t) tvc->f.m.Length, (off_t) B_ASYNC, error);
 	    }
 #endif /* AFS_SGI61_ENV */
 	}
@@ -520,8 +509,8 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t ** avcp, struct fid * fidp)
 {
     struct VenusFid vfid;
     struct vrequest treq;
-    register struct cell *tcell;
-    register afs_int32 code = 0;
+    struct cell *tcell;
+    afs_int32 code = 0;
     afs_int32 ret;
 
 #if defined(AFS_SGI64_ENV) && defined(CKPT) && !defined(_R5000_CVT_WAR)
