@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -32,7 +32,8 @@ static struct afspag_cell *cells = 0;
 static struct afspag_cell *primary_cell = 0;
 
 
-struct afspag_cell *afspag_GetCell(char *acell)
+struct afspag_cell *
+afspag_GetCell(char *acell)
 {
     struct afspag_cell *tcell;
 
@@ -44,10 +45,10 @@ struct afspag_cell *afspag_GetCell(char *acell)
     }
 
     if (!tcell) {
-	tcell = (struct afspag_cell *)afs_osi_Alloc(sizeof(struct afspag_cell));
+	tcell = afs_osi_Alloc(sizeof(struct afspag_cell));
 	if (!tcell)
 	    goto out;
-	tcell->cellname = (char *)afs_osi_Alloc(strlen(acell) + 1);
+	tcell->cellname = afs_osi_Alloc(strlen(acell) + 1);
 	if (!tcell->cellname) {
 	    afs_osi_Free(tcell, sizeof(struct afspag_cell));
 	    tcell = 0;
@@ -66,7 +67,8 @@ out:
 }
 
 
-struct afspag_cell *afspag_GetPrimaryCell()
+struct afspag_cell *
+afspag_GetPrimaryCell(void)
 {
     struct afspag_cell *tcell;
 
@@ -77,7 +79,8 @@ struct afspag_cell *afspag_GetPrimaryCell()
 }
 
 
-void afspag_SetPrimaryCell(char *acell)
+void
+afspag_SetPrimaryCell(char *acell)
 {
     struct afspag_cell *tcell;
 
@@ -88,10 +91,11 @@ void afspag_SetPrimaryCell(char *acell)
 }
 
 
-int afspag_PUnlog(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
+int
+afspag_PUnlog(char *ain, afs_int32 ainSize, afs_ucred_t **acred)
 {
-    register afs_int32 i;
-    register struct unixuser *tu;
+    afs_int32 i;
+    struct unixuser *tu;
     afs_int32 pag, uid;
 
     AFS_STATCNT(PUnlog);
@@ -99,22 +103,29 @@ int afspag_PUnlog(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
 	return EIO;		/* Inappropriate ioctl for device */
 
     pag = PagInCred(*acred);
-    uid = (pag == NOPAG) ? (*acred)->cr_uid : pag;
+    uid = (pag == NOPAG) ? afs_cr_uid(*acred) : pag;
     i = UHash(uid);
     ObtainWriteLock(&afs_xuser, 823);
     for (tu = afs_users[i]; tu; tu = tu->next) {
 	if (tu->uid == uid) {
-	    tu->vid = UNDEFVID;
+	    tu->refCount++;
+	    ReleaseWriteLock(&afs_xuser);
+
+	    afs_LockUser(tu, WRITE_LOCK, 368);
+
 	    tu->states &= ~UHasTokens;
-	    /* security is not having to say you're sorry */
-	    memset((char *)&tu->ct, 0, sizeof(struct ClearToken));
+	    tu->viceId = UNDEFVID;
+	    afs_FreeTokens(&tu->tokens);
 #ifdef UKERNEL
 	    /* set the expire times to 0, causes
 	     * afs_GCUserData to remove this entry
 	     */
-	    tu->ct.EndTimestamp = 0;
 	    tu->tokenTime = 0;
 #endif /* UKERNEL */
+
+	    afs_PutUser(tu, WRITE_LOCK);
+
+	    ObtainWriteLock(&afs_xuser, 369);
 	}
     }
     ReleaseWriteLock(&afs_xuser);
@@ -122,10 +133,11 @@ int afspag_PUnlog(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
 }
 
 
-int afspag_PSetTokens(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
+int
+afspag_PSetTokens(char *ain, afs_int32 ainSize, afs_ucred_t **acred)
 {
     afs_int32 i;
-    register struct unixuser *tu;
+    struct unixuser *tu;
     struct afspag_cell *tcell;
     struct ClearToken clear;
     char *stp;
@@ -172,38 +184,24 @@ int afspag_PSetTokens(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
     if (!tcell) return ESRCH;
     if (set_parent_pag) {
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-#if defined(AFS_DARWIN_ENV)
-	struct proc *p = current_proc();	/* XXX */
-#else
-	struct proc *p = curproc;	/* XXX */
-#endif
-#ifndef AFS_DARWIN80_ENV
-	uprintf("Process %d (%s) tried to change pags in PSetTokens\n",
-		p->p_pid, p->p_comm);
-#endif
-	setpag(p, acred, -1, &pag, 1);
-#else
-#ifdef	AFS_OSF_ENV
-	setpag(u.u_procp, acred, -1, &pag, 1);	/* XXX u.u_procp is a no-op XXX */
+	char procname[256];
+	osi_procname(procname, 256);
+
+	afs_warnuser("Process %d (%s) tried to change pags in PSetTokens\n",
+		     MyPidxx2Pid(MyPidxx), procname);
+	setpag(osi_curproc(), acred, -1, &pag, 1);
 #else
 	setpag(acred, -1, &pag, 1);
-#endif
 #endif
     }
     pag = PagInCred(*acred);
-    uid = (pag == NOPAG) ? (*acred)->cr_uid : pag;
+    uid = (pag == NOPAG) ? afs_cr_uid(*acred) : pag;
     /* now we just set the tokens */
     tu = afs_GetUser(uid, tcell->cellnum, WRITE_LOCK);
     if (!tu->cellinfo)
 	tu->cellinfo = (void *)tcell;
-    tu->vid = clear.ViceId;
-    if (tu->stp != NULL) {
-	afs_osi_Free(tu->stp, tu->stLen);
-    }
-    tu->stp = (char *)afs_osi_Alloc(stLen);
-    tu->stLen = stLen;
-    memcpy(tu->stp, stp, stLen);
-    tu->ct = clear;
+    afs_FreeTokens(&tu->tokens);
+    afs_AddRxkadToken(&tu->tokens, stp, stLen, &clear);
 #ifndef AFS_NOSTATS
     afs_stats_cmfullperf.authent.TicketUpdates++;
     afs_ComputePAGStats();
@@ -223,6 +221,7 @@ SPAGCB_GetCreds(struct rx_call *a_call, afs_int32 a_uid,
                 CredInfos *a_creds)
 {
     struct unixuser *tu;
+    union tokenUnion *token;
     CredInfo *tci;
     int bucket, count, i = 0, clen;
     char *cellname;
@@ -254,8 +253,7 @@ SPAGCB_GetCreds(struct rx_call *a_call, afs_int32 a_uid,
 	return UAESRCH;
     }
 
-    a_creds->CredInfos_val =
-	(CredInfo *)afs_osi_Alloc(count * sizeof(CredInfo));
+    a_creds->CredInfos_val = afs_osi_Alloc(count * sizeof(CredInfo));
     if (!a_creds->CredInfos_val)
 	goto out;
     a_creds->CredInfos_len = count;
@@ -265,30 +263,47 @@ SPAGCB_GetCreds(struct rx_call *a_call, afs_int32 a_uid,
 	if (tu->uid == a_uid && tu->cellinfo &&
 	    (tu->states & UHasTokens) && !(tu->states & UTokensBad)) {
 
+	    tu->refCount++;
+	    ReleaseWriteLock(&afs_xuser);
+
+	    afs_LockUser(tu, READ_LOCK, 0);
+
+	    token = afs_FindToken(tu->tokens, RX_SECIDX_KAD);
+
 	    tci = &a_creds->CredInfos_val[i];
-	    tci->vid               = tu->vid;
-	    tci->ct.AuthHandle     = tu->ct.AuthHandle;
-	    memcpy(tci->ct.HandShakeKey, tu->ct.HandShakeKey, 8);
-	    tci->ct.ViceId         = tu->ct.ViceId;
-	    tci->ct.BeginTimestamp = tu->ct.BeginTimestamp;
-	    tci->ct.EndTimestamp   = tu->ct.EndTimestamp;
+	    tci->vid		   = token->rxkad.clearToken.ViceId;
+	    tci->ct.AuthHandle     = token->rxkad.clearToken.AuthHandle;
+	    memcpy(tci->ct.HandShakeKey,
+		   token->rxkad.clearToken.HandShakeKey, 8);
+	    tci->ct.ViceId         = token->rxkad.clearToken.ViceId;
+	    tci->ct.BeginTimestamp = token->rxkad.clearToken.BeginTimestamp;
+	    tci->ct.EndTimestamp   = token->rxkad.clearToken.EndTimestamp;
 
 	    cellname = ((struct afspag_cell *)(tu->cellinfo))->cellname;
 	    clen = strlen(cellname) + 1;
 	    tci->cellname = afs_osi_Alloc(clen);
-	    if (!tci->cellname)
-		goto out;
-	    memcpy(tci->cellname, cellname, clen);
-
-	    tci->st.st_len = tu->stLen;
-	    tci->st.st_val = afs_osi_Alloc(tu->stLen);
-	    if (!tci->st.st_val) {
-		afs_osi_Free(tci->cellname, clen);
+	    if (!tci->cellname) {
+		afs_PutUser(tu, READ_LOCK);
+		ObtainWriteLock(&afs_xuser, 370);
 		goto out;
 	    }
-	    memcpy(tci->st.st_val, tu->stp, tu->stLen);
+	    memcpy(tci->cellname, cellname, clen);
+
+	    tci->st.st_len = token->rxkad.ticketLen;
+	    tci->st.st_val = afs_osi_Alloc(token->rxkad.ticketLen);
+	    if (!tci->st.st_val) {
+		afs_PutUser(tu, READ_LOCK);
+		afs_osi_Free(tci->cellname, clen);
+		ObtainWriteLock(&afs_xuser, 371);
+		goto out;
+	    }
+	    memcpy(tci->st.st_val,
+		   token->rxkad.ticket, token->rxkad.ticketLen);
 	    if (tu->states & UPrimary)
 		tci->states |= UPrimary;
+
+	    afs_PutUser(tu, READ_LOCK);
+	    ObtainWriteLock(&afs_xuser, 372);
 	}
     }
 
@@ -313,7 +328,8 @@ out:
 }
 
 
-int afspag_PSetSysName(char *ain, afs_int32 ainSize, AFS_UCRED **acred)
+int
+afspag_PSetSysName(char *ain, afs_int32 ainSize, afs_ucred_t **acred)
 {
     int setsysname, count, t;
     char *cp, *setp;
